#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_portal.interfaces import IPortalTemplateContainer, IPortalTemplate, IPortalContext, \
    MANAGE_TEMPLATE_PERMISSION
from pyams_skin.interfaces import IPageHeader, IContentTitle
from pyams_skin.interfaces.viewlet import IWidgetTitleViewletManager
from pyams_skin.layer import IPyAMSLayer
from pyams_zmi.layer import IAdminLayer
from z3c.form.interfaces import IDataExtractedEvent
from zope.component.interfaces import ISite

# import packages
from pyams_form.form import AJAXAddForm
from pyams_pagelet.pagelet import pagelet_config
from pyams_portal.template import PortalTemplate
from pyams_portal.zmi.container import PortalTemplateContainerTable
from pyams_skin.container import delete_container_element
from pyams_skin.page import DefaultPageHeaderAdapter
from pyams_skin.viewlet.toolbar import ToolbarAction
from pyams_utils.adapter import adapter_config
from pyams_utils.registry import query_utility
from pyams_utils.traversing import get_parent
from pyams_utils.unicode import translate_string
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminDialogAddForm
from pyramid.events import subscriber
from pyramid.view import view_config
from z3c.form import field
from zope.interface import Interface, Invalid

from pyams_portal import _


@adapter_config(context=(IPortalTemplate, IPyAMSLayer, Interface), provides=IPageHeader)
class PortalTemplateHeaderAdapter(DefaultPageHeaderAdapter):
    """Portal template header adapter"""

    icon_class = 'fa fa-fw fa-columns'

    @property
    def title(self):
        translate = self.request.localizer.translate
        # check for templates container
        container = get_parent(self.context, IPortalTemplateContainer)
        if container is not None:
            return translate(_("« {0} »  portal template")).format(self.context.name)
        # check for portal context
        context = get_parent(self.context, IPortalContext)
        if context is not None:
            adapter = self.request.registry.queryMultiAdapter((context, self.request, self.view), IContentTitle)
            if adapter is None:
                adapter = IContentTitle(context, None)
            if adapter is not None:
                return adapter.title


#
# Template views
#

@viewlet_config(name='add-portal-template.action', context=ISite, layer=IAdminLayer,
                view=PortalTemplateContainerTable, manager=IWidgetTitleViewletManager,
                permission=MANAGE_TEMPLATE_PERMISSION, weight=1)
class PortalTemplateAddAction(ToolbarAction):
    """Portal template add action"""

    label = _("Add template")
    url = 'add-portal-template.html'
    modal_target = True


@pagelet_config(name='add-portal-template.html', context=ISite, layer=IPyAMSLayer,
                permission=MANAGE_TEMPLATE_PERMISSION)
class PortalTemplateAddForm(AdminDialogAddForm):
    """Portal template add form"""

    title = _("Portal templates")
    legend = _("Add shared template")
    icon_css_class = 'fa fa-fw fa-columns'

    fields = field.Fields(IPortalTemplate)
    ajax_handler = 'add-portal-template.json'
    edit_permission = None

    def create(self, data):
        return PortalTemplate()

    def add(self, template):
        context = query_utility(IPortalTemplateContainer)
        context[translate_string(template.name, spaces='-')] = template

    def nextURL(self):
        return absolute_url(self.context, self.request, 'portal-templates.html')


@subscriber(IDataExtractedEvent, form_selector=PortalTemplateAddForm)
def handle_new_template_data_extraction(event):
    """Handle new template form data extraction"""
    container = query_utility(IPortalTemplateContainer)
    name = event.data.get('name')
    if name in container:
        event.form.widgets.errors += (Invalid(_("Specified name is already used!")),)


@view_config(name='add-portal-template.json', context=ISite, request_type=IPyAMSLayer,
             permission=MANAGE_TEMPLATE_PERMISSION, renderer='json', xhr=True)
class PortalTemplateAJAXAddForm(AJAXAddForm, PortalTemplateAddForm):
    """Portal template add form, AJAX handler"""
